/* Copyright 2009 Fog Creek Software, Inc. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Data;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * the following DLL files contained in C:\Program Files\FogBugz\Website\bin\ 
 * FogBugz.dll, FogCreek.Plugins.dll, FogCreek.Plugins.InterfaceEvents.dll     */
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Entity;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz;
using FogCreek.FogBugz.UI;
using FogCreek.FogBugz.Database;
using FogCreek.FogBugz.Search;
using FogCreek.FogBugz.Database.Entity;

namespace IPluginSearchAxis_Example
{
    /* Class Declaration: Inherit from Plugin, expose IPluginSearchAxis */
    public class IPluginSearchAxis_Example : Plugin, IPluginDatabase, IPluginBugJoin,
        IPluginGridColumn, IPluginSearchAxis, IPluginFullTextIndexBug
    {
        private const string STABLENAME = "Kiwi";
        private string sPrefixedTableName;
        /* PluginID is the same as you set in AssemblyInfo.cs */
        private const string PLUGIN_ID = "IPluginSearchAxis_Example@fogcreek.com";
        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable. */
        public IPluginSearchAxis_Example(CPluginApi api)
            : base(api)
        {
            /* database tables created by plugins are prefixed by a unique identifier.
             * To use the table directly in a query, you need the full name */
            sPrefixedTableName = api.Database.PluginTableName(STABLENAME);
        }

        #region IPluginDatabase Members

        public CTable[] DatabaseSchema()
        {
            CTable kiwiTable = api.Database.NewTable(api.Database.PluginTableName(STABLENAME));

            kiwiTable.sDesc = "A kiwi for every case.";
            kiwiTable.AddVarcharColumn("sName", 255, false);
            kiwiTable.AddDateColumn("dtBirth", false);
            kiwiTable.AddIntColumn("ixBug", true, 2);
            kiwiTable.AddAutoIncrementPrimaryKey("ixKiwi");
            return new CTable[] { kiwiTable };
        }

        public int DatabaseSchemaVersion()
        {
            return 2;
        }

        /* this is the first version: no upgrade */
        public void DatabaseUpgradeAfter(int ixVersionFrom, int ixVersionTo,
                                         CDatabaseUpgradeApi apiUpgrade) { }

        public void DatabaseUpgradeBefore(int ixVersionFrom, int ixVersionTo,
                                          CDatabaseUpgradeApi apiUpgrade) { }

        #endregion

        #region IPluginBugJoin Members

        public string[] BugJoinTables()
        {
            return new string[] { STABLENAME };
        }

        #endregion

        #region IPluginGridColumn Members

        public CGridColumn[] GridColumns()
        {
            CGridColumn gridCol1 = api.Grid.CreateGridColumn();
            gridCol1.sName = "Kiwi Name";
            gridCol1.sTitle = "Kiwi Name";
            gridCol1.iType = 0;

            CGridColumn gridCol2 = api.Grid.CreateGridColumn();
            gridCol2.sName = "Kiwi Birthdate";
            gridCol2.sTitle = "Kiwi Birthdate";
            gridCol2.iType = 1;

            return new CGridColumn[] { gridCol1, gridCol2 };
        }

        public CBugQuery GridColumnQuery(CGridColumn col)
        {
            /* Return a CBugQuery with the data you need joined
             * in. If your table is already joined to bug in
             * IPluginBugJoin, FogBugz does the work for you. */
            return api.Bug.NewBugQuery();
        }

        public string[] GridColumnDisplay(CGridColumn col,
                                          CBug[] rgBug,
                                          bool fPlainText)
        {
            string sTableColumn = "sName";
            switch (col.iType)
            {
                case 0:
                    sTableColumn = "sName";
                    break;
                case 1:
                    sTableColumn = "dtBirth";
                    break;
            }
            string[] sValues = new string[rgBug.Length];

            for (int i = 0; i < rgBug.Length; i++)
            {
                /* For tables joined in IPluginBugJoin, use
                 * GetPluginField to fetch the values you need
                 * for the GridColumn. */
                object pluginField = rgBug[i].GetPluginField(PLUGIN_ID, string.Format("{0}", sTableColumn));
                sValues[i] = (pluginField == null) ?
                             "" :
                             HttpUtility.HtmlEncode(pluginField.ToString());
            }
            return sValues;
        }

        public CBugQuery GridColumnSortQuery(CGridColumn col, bool fDescending,
                                                bool fIncludeSelect)
        {
            string sTableColumn = "sName";
            switch (col.iType)
            {
                case 0:
                    sTableColumn = "sName";
                    break;
                case 1:
                    sTableColumn = "dtBirth";
                    break;
            }
            /* Return a CBugQuery with the data you need joined
             * in and sorted appropriately. Include an explicit
             * select if fIncludeSelect is true. If your table is
             * already joined to bug in IPluginBugJoin, FogBugz
             * does the work for you, ignore fIncludeSelect. */
            CBugQuery bugQuery = api.Bug.NewBugQuery();
            bugQuery.AddOrderBy(string.Format("{0}.{1} {2}",
                                            sPrefixedTableName,
                                            sTableColumn,
                                            (fDescending ? "DESC" : "ASC")
                                            )
                             );
            return bugQuery;
        }

        #endregion
    
        #region IPluginSearchAxis Members

        public bool  SearchAxisIsFullText(string sAxis, SearchType nType, string sValue)
        {
            if (nType != SearchType.Bug)
                return false;
            switch (sAxis.ToLower())
            {
                /* use full text search on the kiwiname axis */
                case "kiwiname":
                    return true;
                /* use axis (SQL) search for the date axis, kiwibirth */
                case "kiwibirth":
                    return false;
            }
            return false;
        }

        public CSelectQuery SearchAxisQuery(string sAxis, SearchType nType, string sValue)
        {
            /* only return results for cases, not wiki pages or discussion topics */
            if (nType != SearchType.Bug)
                return null;

            switch (sAxis.ToLower())
            {
                case "kiwiname":
                    return null;
                case "kiwibirth":
                    /* Like GridColumn, search axis queries should be on the Bug table.
                     * They should select the ixBug of the results */
                    CSelectQuery select = api.Database.NewSelectQuery("Bug");
                    select.AddLeftJoin(sPrefixedTableName,
                                       string.Format("Bug.ixBug = {0}.ixBug", sPrefixedTableName));
                    select.AddSelect("Bug.ixBug");
                    
                    /* FogBugz built-in datetime parsing is not currently exposed to plugins.
                     * Perform your own parsing or use SQL servers */
                    DateTime dt;
                    if (DateTime.TryParse(sValue, out dt))
                    {
                        select.AddWhere(string.Format("{0}.dtBirth = '{1}'",
                                                      sPrefixedTableName,
                                                      sValue));
                    }
                    else
                    {
                        /* If parsing failed, inform the user */
                        api.Notifications.AddError(string.Format("Invalid date: {0}",
                                                                 HttpUtility.HtmlEncode(sValue))
                                                  );
                        /* and return no results */
                        select.AddWhere("1 = 0");
                    }
                    return select;
                default:
                    /* Return null for unhandled axes */
                    return null;
            }
        }

        #endregion

        #region IPluginFullTextIndexBug Members

        public void FullTextIndexBug(CBug bug, CBugIndex document)
        {
            /* FullTextIndexBug is called upon the entry of new cases, and when
             * the search index is reset. The bug in question is passed as a CBug
             * Add your fields to the BugIndex document. Make sure to handle
             * null values for your joined fields */
            bug.IgnorePermissions = true;
            string sName = (bug.GetPluginField(PLUGIN_ID, "sName") == null) ?
                "" : bug.GetPluginField(PLUGIN_ID, "sName").ToString();
            string sdtBirth = (bug.GetPluginField(PLUGIN_ID, "dtBirth") == null) ?
                "" : bug.GetPluginField(PLUGIN_ID, "dtBirth").ToString();
            bug.IgnorePermissions = false;
            document.AddText("kiwiname", sName);
            document.AddText("kiwibirth", sdtBirth);
        }

        #endregion
    }
}

